@@ -80,12 +80,16 @@ LOG = log.getLogger(__name__)
 
 class NovaLikeServer(object):
     def __init__(self, **kwargs):
+        self.id = kwargs.pop('id')
         for k, v in kwargs.items():
             setattr(self, k, v)
 
     def __repr__(self):
         return '<NovaLikeServer: %s>' % getattr(self, 'name', 'unknown-name')
 
+    def __eq__(self, other):
+        return self.id == other.id
+
 
 class InstanceDiscovery(plugin_base.DiscoveryBase):
     method = None
@@ -117,6 +121,10 @@ class InstanceDiscovery(plugin_base.DiscoveryBase):
             self._connection = libvirt_utils.get_libvirt_connection(self.conf)
         return self._connection
 
+    @connection.setter
+    def connection(self, value):
+        self._connection = value
+
     def discover(self, manager, param=None):
         """Discover resources to monitor."""
         if self.method != "libvirt_metadata":
