@@ -44,6 +44,10 @@ class LibvirtInspector(virt_inspector.Inspector):
             self._connection = libvirt_utils.get_libvirt_connection(self.conf)
         return self._connection
 
+    @connection.setter
+    def connection(self, value):
+        self._connection = value
+
     @libvirt_utils.retry_on_disconnect
     def _lookup_by_uuid(self, instance):
         instance_name = util.instance_name(instance)
@@ -71,10 +75,30 @@ class LibvirtInspector(virt_inspector.Inspector):
         domain = self._get_domain_not_shut_off_or_raise(instance)
         # TODO(gordc): this can probably be cached since it can be used to get
         # all data related
-        stats = self.connection.domainListGetStats([domain])
-        dom_stat = stats[0][1]
-        return virt_inspector.CPUStats(number=dom_stat['vcpu.current'],
-                                       time=dom_stat['cpu.time'])
+        stats = self.connection.domainListGetStats([domain], 0)[0][1]
+        cpu_time = 0
+        current_cpus = stats.get('vcpu.current')
+        # Iterate over the maximum number of CPUs here, and count the
+        # actual number encountered, since the vcpu.x structure can
+        # have holes according to
+        # https://libvirt.org/git/?p=libvirt.git;a=blob;f=src/libvirt-domain.c
+        # virConnectGetAllDomainStats()
+        for vcpu in six.moves.range(stats.get('vcpu.maximum', 0)):
+            try:
+                cpu_time += (stats.get('vcpu.%s.time' % vcpu) +
+                             stats.get('vcpu.%s.wait' % vcpu))
+                current_cpus -= 1
+            except TypeError:
+                # pass here, if there are too many holes, the cpu count will
+                # not match, so don't need special error handling.
+                pass
+
+        if current_cpus:
+            # There wasn't enough data, so fall back
+            cpu_time = stats.get('cpu.time')
+
+        return virt_inspector.CPUStats(number=stats['vcpu.current'],
+                                       time=cpu_time)
 
     def inspect_cpu_l3_cache(self, instance):
         domain = self._lookup_by_uuid(instance)
@@ -142,12 +166,12 @@ class LibvirtInspector(virt_inspector.Inspector):
             dom_stats = domain.interfaceStats(name)
             stats = virt_inspector.InterfaceStats(rx_bytes=dom_stats[0],
                                                   rx_packets=dom_stats[1],
-                                                  rx_drop=dom_stats[2],
-                                                  rx_errors=dom_stats[3],
+                                                  rx_errors=dom_stats[2],
+                                                  rx_drop=dom_stats[3],
                                                   tx_bytes=dom_stats[4],
                                                   tx_packets=dom_stats[5],
-                                                  tx_drop=dom_stats[6],
-                                                  tx_errors=dom_stats[7])
+                                                  tx_errors=dom_stats[6],
+                                                  tx_drop=dom_stats[7])
             yield (interface, stats)
 
     def inspect_disks(self, instance):
