@@ -17,13 +17,13 @@
 
 from oslo_config import cfg
 from oslo_utils import units
-from oslo_vmware import api
 import six
 
 from ceilometer.compute.virt import inspector as virt_inspector
 from ceilometer.compute.virt.vmware import vsphere_operations
 from ceilometer.i18n import _
 
+vmware_api = None
 
 opt_group = cfg.OptGroup(name='vmware',
                          title='Options for VMware')
@@ -65,9 +65,6 @@ OPTS = [
                     'work-arounds.'),
 ]
 
-cfg.CONF.register_group(opt_group)
-cfg.CONF.register_opts(OPTS, group=opt_group)
-
 VC_AVERAGE_MEMORY_CONSUMED_CNTR = 'mem:consumed:average'
 VC_AVERAGE_CPU_CONSUMED_CNTR = 'cpu:usage:average'
 VC_NETWORK_RX_COUNTER = 'net:received:average'
@@ -78,36 +75,51 @@ VC_DISK_WRITE_RATE_CNTR = "disk:write:average"
 VC_DISK_WRITE_REQUESTS_RATE_CNTR = "disk:numberWriteAveraged:average"
 
 
-def get_api_session():
-    api_session = api.VMwareAPISession(
-        cfg.CONF.vmware.host_ip,
-        cfg.CONF.vmware.host_username,
-        cfg.CONF.vmware.host_password,
-        cfg.CONF.vmware.api_retry_count,
-        cfg.CONF.vmware.task_poll_interval,
-        wsdl_loc=cfg.CONF.vmware.wsdl_location,
-        port=cfg.CONF.vmware.host_port,
-        cacert=cfg.CONF.vmware.ca_file,
-        insecure=cfg.CONF.vmware.insecure)
+def get_api_session(conf):
+    global vmware_api
+    if vmware_api is None:
+        vmware_api = __import__('oslo_vmware.api')
+    api_session = vmware_api.VMwareAPISession(
+        conf.vmware.host_ip,
+        conf.vmware.host_username,
+        conf.vmware.host_password,
+        conf.vmware.api_retry_count,
+        conf.vmware.task_poll_interval,
+        wsdl_loc=conf.vmware.wsdl_location,
+        port=conf.vmware.host_port,
+        cacert=conf.vmware.ca_file,
+        insecure=conf.vmware.insecure)
     return api_session
 
 
 class VsphereInspector(virt_inspector.Inspector):
 
-    def __init__(self):
-        super(VsphereInspector, self).__init__()
+    def __init__(self, conf):
+        super(VsphereInspector, self).__init__(conf)
         self._ops = vsphere_operations.VsphereOperations(
-            get_api_session(), 1000)
+            get_api_session(self.conf), 1000)
 
-    def inspect_cpu_util(self, instance, duration=None):
-        vm_moid = self._ops.get_vm_moid(instance.id)
-        if vm_moid is None:
+    def _get_vm_mobj_not_power_off_or_raise(self, instance):
+        vm_mobj = self._ops.get_vm_mobj(instance.id)
+
+        if vm_mobj is None:
             raise virt_inspector.InstanceNotFoundException(
                 _('VM %s not found in VMware vSphere') % instance.id)
+
+        vm_powerState = self._ops.query_vm_property(vm_mobj,
+                                                    'runtime.powerState')
+        if vm_powerState == "poweredOff":
+            raise virt_inspector.InstanceShutOffException(
+                _('VM %s is poweroff in VMware vSphere') % instance.id)
+
+        return vm_mobj
+
+    def inspect_cpu_util(self, instance, duration=None):
+        vm_mobj = self._get_vm_mobj_not_power_off_or_raise(instance)
         cpu_util_counter_id = self._ops.get_perf_counter_id(
             VC_AVERAGE_CPU_CONSUMED_CNTR)
         cpu_util = self._ops.query_vm_aggregate_stats(
-            vm_moid, cpu_util_counter_id, duration)
+            vm_mobj, cpu_util_counter_id, duration)
 
         # For this counter vSphere returns values scaled-up by 100, since the
         # corresponding API can't return decimals, but only longs.
@@ -117,10 +129,7 @@ class VsphereInspector(virt_inspector.Inspector):
         return virt_inspector.CPUUtilStats(util=cpu_util)
 
     def inspect_vnic_rates(self, instance, duration=None):
-        vm_moid = self._ops.get_vm_moid(instance.id)
-        if not vm_moid:
-            raise virt_inspector.InstanceNotFoundException(
-                _('VM %s not found in VMware vSphere') % instance.id)
+        vm_mobj = self._get_vm_mobj_not_power_off_or_raise(instance)
 
         vnic_stats = {}
         vnic_ids = set()
@@ -128,7 +137,14 @@ class VsphereInspector(virt_inspector.Inspector):
         for net_counter in (VC_NETWORK_RX_COUNTER, VC_NETWORK_TX_COUNTER):
             net_counter_id = self._ops.get_perf_counter_id(net_counter)
             vnic_id_to_stats_map = self._ops.query_vm_device_stats(
-                vm_moid, net_counter_id, duration)
+                vm_mobj, net_counter_id, duration)
+            # The sample for this map is: {4000: 0.0, vmnic5: 0.0, vmnic4: 0.0,
+            #               vmnic3: 0.0, vmnic2: 0.0, vmnic1: 0.0, vmnic0: 0.0}
+            # "4000" is the virtual nic which we need.
+            # And these "vmnic*" are phynical nics in the host, so we remove it
+            vnic_id_to_stats_map = {k: v for (k, v)
+                                    in vnic_id_to_stats_map.items()
+                                    if not k.startswith('vmnic')}
             vnic_stats[net_counter] = vnic_id_to_stats_map
             vnic_ids.update(six.iterkeys(vnic_id_to_stats_map))
 
@@ -149,23 +165,17 @@ class VsphereInspector(virt_inspector.Inspector):
             yield (interface, stats)
 
     def inspect_memory_usage(self, instance, duration=None):
-        vm_moid = self._ops.get_vm_moid(instance.id)
-        if vm_moid is None:
-            raise virt_inspector.InstanceNotFoundException(
-                _('VM %s not found in VMware vSphere') % instance.id)
+        vm_mobj = self._get_vm_mobj_not_power_off_or_raise(instance)
         mem_counter_id = self._ops.get_perf_counter_id(
             VC_AVERAGE_MEMORY_CONSUMED_CNTR)
         memory = self._ops.query_vm_aggregate_stats(
-            vm_moid, mem_counter_id, duration)
+            vm_mobj, mem_counter_id, duration)
         # Stat provided from vSphere is in KB, converting it to MB.
         memory = memory / units.Ki
         return virt_inspector.MemoryUsageStats(usage=memory)
 
     def inspect_disk_rates(self, instance, duration=None):
-        vm_moid = self._ops.get_vm_moid(instance.id)
-        if not vm_moid:
-            raise virt_inspector.InstanceNotFoundException(
-                _('VM %s not found in VMware vSphere') % instance.id)
+        vm_mobj = self._get_vm_mobj_not_power_off_or_raise(instance)
 
         disk_stats = {}
         disk_ids = set()
@@ -179,7 +189,7 @@ class VsphereInspector(virt_inspector.Inspector):
         for disk_counter in disk_counters:
             disk_counter_id = self._ops.get_perf_counter_id(disk_counter)
             disk_id_to_stat_map = self._ops.query_vm_device_stats(
-                vm_moid, disk_counter_id, duration)
+                vm_mobj, disk_counter_id, duration)
             disk_stats[disk_counter] = disk_id_to_stat_map
             disk_ids.update(six.iterkeys(disk_id_to_stat_map))
 
