@@ -51,7 +51,30 @@ class TestLibvirtInspection(base.BaseTestCase):
         self.domain = mock.Mock()
         self.addCleanup(mock.patch.stopall)
 
-    def test_inspect_cpus(self):
+    def test_inspect_cpus_stats(self):
+        fake_stats = [({}, {'vcpu.current': 2,
+                            'vcpu.maximum': 4,
+                            'vcpu.0.time': 10000,
+                            'vcpu.2.time': 10000,
+                            'vcpu.0.wait': 10000,
+                            'vcpu.2.wait': 10000,
+                            'cpu.time': 999999})]
+
+        with contextlib.ExitStack() as stack:
+            stack.enter_context(mock.patch.object(self.inspector.connection,
+                                                  'lookupByUUIDString',
+                                                  return_value=self.domain))
+            stack.enter_context(mock.patch.object(self.domain, 'info',
+                                                  return_value=(0, 0, 0,
+                                                                None, None)))
+            stack.enter_context(mock.patch.object(self.inspector.connection,
+                                                  'domainListGetStats',
+                                                  return_value=fake_stats))
+            cpu_info = self.inspector.inspect_cpus(self.instance)
+            self.assertEqual(2, cpu_info.number)
+            self.assertEqual(40000, cpu_info.time)
+
+    def test_inspect_cpus_stats_fallback_cpu_time(self):
         fake_stats = [({}, {'cpu.time': 999999, 'vcpu.current': 2})]
         with contextlib.ExitStack() as stack:
             stack.enter_context(mock.patch.object(self.inspector.connection,
@@ -150,9 +173,9 @@ class TestLibvirtInspection(base.BaseTestCase):
         """
 
         interface_stats = {
-            'vnet0': (1, 2, 0, 0, 3, 4, 0, 0),
-            'vnet1': (5, 6, 0, 0, 7, 8, 0, 0),
-            'vnet2': (9, 10, 0, 0, 11, 12, 0, 0),
+            'vnet0': (1, 2, 21, 22, 3, 4, 23, 24),
+            'vnet1': (5, 6, 25, 26, 7, 8, 27, 28),
+            'vnet2': (9, 10, 29, 30, 11, 12, 31, 32),
         }
         interfaceStats = interface_stats.__getitem__
 
@@ -184,6 +207,10 @@ class TestLibvirtInspection(base.BaseTestCase):
             self.assertEqual(2, info0.rx_packets)
             self.assertEqual(3, info0.tx_bytes)
             self.assertEqual(4, info0.tx_packets)
+            self.assertEqual(21, info0.rx_errors)
+            self.assertEqual(22, info0.rx_drop)
+            self.assertEqual(23, info0.tx_errors)
+            self.assertEqual(24, info0.tx_drop)
 
             vnic1, info1 = interfaces[1]
             self.assertEqual('vnet1', vnic1.name)
@@ -197,6 +224,10 @@ class TestLibvirtInspection(base.BaseTestCase):
             self.assertEqual(6, info1.rx_packets)
             self.assertEqual(7, info1.tx_bytes)
             self.assertEqual(8, info1.tx_packets)
+            self.assertEqual(25, info1.rx_errors)
+            self.assertEqual(26, info1.rx_drop)
+            self.assertEqual(27, info1.tx_errors)
+            self.assertEqual(28, info1.tx_drop)
 
             vnic2, info2 = interfaces[2]
             self.assertEqual('vnet2', vnic2.name)
@@ -207,6 +238,10 @@ class TestLibvirtInspection(base.BaseTestCase):
             self.assertEqual(10, info2.rx_packets)
             self.assertEqual(11, info2.tx_bytes)
             self.assertEqual(12, info2.tx_packets)
+            self.assertEqual(29, info2.rx_errors)
+            self.assertEqual(30, info2.rx_drop)
+            self.assertEqual(31, info2.tx_errors)
+            self.assertEqual(32, info2.tx_drop)
 
     def test_inspect_vnics_with_domain_shutoff(self):
         connection = self.inspector.connection
