@@ -25,6 +25,7 @@ from ceilometer import declarative
 from ceilometer.hardware.inspector import base as inspector_base
 from ceilometer.hardware.pollsters import generic
 from ceilometer import sample
+from ceilometer import service
 from ceilometer.tests import base as test_base
 
 
@@ -44,8 +45,8 @@ class TestMeterDefinition(test_base.BaseTestCase):
         cfg = dict(name='test', type='gauge')
         try:
             generic.MeterDefinition(cfg)
-        except generic.MeterDefinitionException as e:
-            self.assertEqual("Missing field unit", e.message)
+        except declarative.MeterDefinitionException as e:
+            self.assertEqual("Missing field unit", e.brief_message)
 
     def test_config_invalid_field(self):
         cfg = dict(name='test',
@@ -62,8 +63,9 @@ class TestMeterDefinition(test_base.BaseTestCase):
                    snmp_inspector={})
         try:
             generic.MeterDefinition(cfg)
-        except generic.MeterDefinitionException as e:
-            self.assertEqual("Unrecognized type value invalid", e.message)
+        except declarative.MeterDefinitionException as e:
+            self.assertEqual("Unrecognized type value invalid",
+                             e.brief_message)
 
     @mock.patch('ceilometer.hardware.pollsters.generic.LOG')
     def test_bad_metric_skip(self, LOG):
@@ -82,7 +84,7 @@ class TestMeterDefinition(test_base.BaseTestCase):
         data = generic.load_definition(cfg)
         self.assertEqual(2, len(data))
         LOG.error.assert_called_with(
-            "Error loading meter definition : "
+            "Error loading meter definition: %s",
             "Unrecognized type value invalid")
 
 
@@ -108,15 +110,15 @@ class TestGenericPollsters(test_base.BaseTestCase):
 
     def setUp(self):
         super(TestGenericPollsters, self).setUp()
-        self.conf = self.useFixture(fixture_config.Config()).conf
+        conf = service.prepare_service([], [])
+        self.conf = self.useFixture(fixture_config.Config(conf)).conf
         self.resources = ["snmp://test", "snmp://test2"]
         self.useFixture(mockpatch.Patch(
             'ceilometer.hardware.inspector.get_inspector',
             self.faux_get_inspector))
-        self.conf(args=[])
-        self.pollster = generic.GenericHardwareDeclarativePollster()
+        self.pollster = generic.GenericHardwareDeclarativePollster(self.conf)
 
-    def __setup_meter_def_file(self, cfg):
+    def _setup_meter_def_file(self, cfg):
         if six.PY3:
             cfg = cfg.encode('utf-8')
         meter_cfg_file = fileutils.write_to_tempfile(content=cfg,
@@ -126,7 +128,7 @@ class TestGenericPollsters(test_base.BaseTestCase):
             'meter_definitions_file',
             meter_cfg_file, group='hardware')
         cfg = declarative.load_definitions(
-            {}, self.conf.hardware.meter_definitions_file)
+            self.conf, {}, self.conf.hardware.meter_definitions_file)
         return cfg
 
     def _check_get_samples(self, name, definition,
@@ -174,11 +176,11 @@ class TestGenericPollsters(test_base.BaseTestCase):
                         name='hardware.test2.abc',
                         unit='process',
                         snmp_inspector=param)]})
-        self.__setup_meter_def_file(meter_cfg)
+        self._setup_meter_def_file(meter_cfg)
         pollster = generic.GenericHardwareDeclarativePollster
         # Clear cached mapping
         pollster.mapping = None
-        exts = pollster.get_pollsters_extensions()
+        exts = pollster.get_pollsters_extensions(self.conf)
         self.assertEqual(2, len(exts))
         self.assertIn(exts[0].name, ['hardware.test1', 'hardware.test2.abc'])
         self.assertIn(exts[1].name, ['hardware.test1', 'hardware.test2.abc'])
