@@ -49,6 +49,28 @@ NOTIFICATION = {
     'publisher_id': "foo123"
 }
 
+USER_META = {
+    'event_type': u'test.create',
+    'timestamp': u'2015-06-1909: 19: 35.786893',
+    'payload': {u'user_id': u'e1d870e51c7340cb9d555b15cbfcaec2',
+                u'resource_id': u'bea70e51c7340cb9d555b15cbfcaec23',
+                u'timestamp': u'2015-06-19T09:19:35.785330',
+                u'created_at': u'2015-06-19T09:25:35.785330',
+                u'launched_at': u'2015-06-19T09:25:40.785330',
+                u'message_signature': u'fake_signature1',
+                u'resource_metadata': {u'foo': u'bar'},
+                u'source': u'30be1fc9a03c4e94ab05c403a8a377f2: openstack',
+                u'volume': 1.0,
+                u'project_id': u'30be1fc9a03c4e94ab05c403a8a377f2',
+                u'metadata': {u'metering.xyz': u'abc', u'ignore': u'this'},
+                },
+    u'_context_tenant': u'30be1fc9a03c4e94ab05c403a8a377f2',
+    u'_context_request_id': u'req-da91b4bf-d2b5-43ae-8b66-c7752e72726d',
+    u'_context_user': u'e1d870e51c7340cb9d555b15cbfcaec2',
+    'message_id': u'939823de-c242-45a2-a399-083f4d6a8c3e',
+    'publisher_id': "foo123"
+}
+
 MIDDLEWARE_EVENT = {
     u'_context_request_id': u'req-a8bfa89b-d28b-4b95-9e4b-7d7875275650',
     u'_context_quota_class': None,
@@ -228,7 +250,7 @@ class TestMeterDefinition(test.BaseTestCase):
                    volume="$.payload.volume",
                    resource_id="$.payload.resource_id",
                    project_id="$.payload.project_id")
-        handler = notifications.MeterDefinition(cfg, mock.Mock())
+        handler = notifications.MeterDefinition(cfg, mock.Mock(), mock.Mock())
         self.assertTrue(handler.match_type("test.create"))
         sample = list(handler.to_samples(NOTIFICATION))[0]
         self.assertEqual(1.0, sample["volume"])
@@ -240,31 +262,32 @@ class TestMeterDefinition(test.BaseTestCase):
     def test_config_required_missing_fields(self):
         cfg = dict()
         try:
-            notifications.MeterDefinition(cfg, mock.Mock())
+            notifications.MeterDefinition(cfg, mock.Mock(), mock.Mock())
         except declarative.DefinitionException as e:
-            self.assertEqual("Required fields ['name', 'type', 'event_type',"
-                             " 'unit', 'volume', 'resource_id']"
-                             " not specified",
-                             encodeutils.exception_to_unicode(e))
+            self.assertIn("Required fields ['name', 'type', 'event_type',"
+                          " 'unit', 'volume', 'resource_id']"
+                          " not specified",
+                          encodeutils.exception_to_unicode(e))
 
     def test_bad_type_cfg_definition(self):
         cfg = dict(name="test", type="foo", event_type="bar.create",
                    unit="foo", volume="bar",
                    resource_id="bea70e51c7340cb9d555b15cbfcaec23")
         try:
-            notifications.MeterDefinition(cfg, mock.Mock())
+            notifications.MeterDefinition(cfg, mock.Mock(), mock.Mock())
         except declarative.DefinitionException as e:
-            self.assertEqual("Invalid type foo specified",
-                             encodeutils.exception_to_unicode(e))
+            self.assertIn("Invalid type foo specified",
+                          encodeutils.exception_to_unicode(e))
 
 
 class TestMeterProcessing(test.BaseTestCase):
 
     def setUp(self):
         super(TestMeterProcessing, self).setUp()
-        self.CONF = self.useFixture(fixture_config.Config()).conf
-        ceilometer_service.prepare_service(argv=[], config_files=[])
-        self.handler = notifications.ProcessMeterNotifications(mock.Mock())
+        conf = ceilometer_service.prepare_service(argv=[], config_files=[])
+        self.CONF = self.useFixture(fixture_config.Config(conf)).conf
+        self.handler = notifications.ProcessMeterNotifications(
+            mock.Mock(conf=self.CONF))
 
     def test_fallback_meter_path(self):
         self.CONF.set_override('meter_definitions_cfg_file',
@@ -312,9 +335,9 @@ class TestMeterProcessing(test.BaseTestCase):
                              project_id="$.payload.project_id")]})
         self._load_meter_def_file(cfg)
         self.assertEqual(2, len(self.handler.definitions))
-        LOG.error.assert_called_with(
-            "Error loading meter definition : "
-            "Invalid type bad_type specified")
+        args, kwargs = LOG.error.call_args_list[0]
+        self.assertEqual("Error loading meter definition: %s", args[0])
+        self.assertTrue(args[1].endswith("Invalid type bad_type specified"))
 
     def test_jsonpath_values_parsed(self):
         cfg = yaml.dump(
@@ -493,6 +516,41 @@ class TestMeterProcessing(test.BaseTestCase):
                 'dict': NOTIFICATION['payload']['resource_metadata']}
         self.assertEqual(meta, s1['resource_metadata'])
 
+    def test_user_meta(self):
+        cfg = yaml.dump(
+            {'metric': [dict(name="test1",
+                        event_type="test.*",
+                        type="delta",
+                        unit="B",
+                        volume="$.payload.volume",
+                        resource_id="$.payload.resource_id",
+                        project_id="$.payload.project_id",
+                        user_metadata="$.payload.metadata",)]})
+        self._load_meter_def_file(cfg)
+        c = list(self.handler.process_notification(USER_META))
+        self.assertEqual(1, len(c))
+        s1 = c[0].as_dict()
+        meta = {'user_metadata': {'xyz': 'abc'}}
+        self.assertEqual(meta, s1['resource_metadata'])
+
+    def test_user_meta_and_custom(self):
+        cfg = yaml.dump(
+            {'metric': [dict(name="test1",
+                        event_type="test.*",
+                        type="delta",
+                        unit="B",
+                        volume="$.payload.volume",
+                        resource_id="$.payload.resource_id",
+                        project_id="$.payload.project_id",
+                        user_metadata="$.payload.metadata",
+                        metadata={'proj': '$.payload.project_id'})]})
+        self._load_meter_def_file(cfg)
+        c = list(self.handler.process_notification(USER_META))
+        self.assertEqual(1, len(c))
+        s1 = c[0].as_dict()
+        meta = {'user_metadata': {'xyz': 'abc'}, 'proj': s1['project_id']}
+        self.assertEqual(meta, s1['resource_metadata'])
+
     def test_multi_match_event_meter(self):
         cfg = yaml.dump(
             {'metric': [dict(name="test1",
