@@ -40,10 +40,8 @@ class BaseConversionTransformer(transformer.TransformerBase):
                        unit and scaling factor (a missing value
                        connotes no change)
         """
-        source = source or {}
-        target = target or {}
-        self.source = source
-        self.target = target
+        self.source = source or {}
+        self.target = target or {}
         super(BaseConversionTransformer, self).__init__(**kwargs)
 
     def _map(self, s, attr):
@@ -72,7 +70,7 @@ class DeltaTransformer(BaseConversionTransformer):
         self.growth_only = growth_only
         self.cache = {}
 
-    def handle_sample(self, context, s):
+    def handle_sample(self, s):
         """Handle a sample, converting if necessary."""
         key = s.name + s.resource_id
         prev = self.cache.get(key)
@@ -130,6 +128,7 @@ class ScalingTransformer(BaseConversionTransformer):
         super(ScalingTransformer, self).__init__(source=source, target=target,
                                                  **kwargs)
         self.scale = self.target.get('scale')
+        self.max = self.target.get('max')
         LOG.debug('scaling conversion transformer with source:'
                   ' %(source)s target: %(target)s:', {'source': self.source,
                                                       'target': self.target})
@@ -147,11 +146,12 @@ class ScalingTransformer(BaseConversionTransformer):
 
     def _convert(self, s, growth=1):
         """Transform the appropriate sample fields."""
+        volume = self._scale(s) * growth
         return sample.Sample(
             name=self._map(s, 'name'),
             unit=self._map(s, 'unit'),
             type=self.target.get('type', s.type),
-            volume=self._scale(s) * growth,
+            volume=min(volume, self.max) if self.max else volume,
             user_id=s.user_id,
             project_id=s.project_id,
             resource_id=s.resource_id,
@@ -159,7 +159,7 @@ class ScalingTransformer(BaseConversionTransformer):
             resource_metadata=s.resource_metadata
         )
 
-    def handle_sample(self, context, s):
+    def handle_sample(self, s):
         """Handle a sample, converting if necessary."""
         LOG.debug('handling sample %s', s)
         if self.source.get('unit', s.unit) == s.unit:
@@ -171,7 +171,7 @@ class ScalingTransformer(BaseConversionTransformer):
 class RateOfChangeTransformer(ScalingTransformer):
     """Transformer based on the rate of change of a sample volume.
 
-    For example taking the current and previous volumes of a cumulative sample
+    For example, taking the current and previous volumes of a cumulative sample
     and producing a gauge value based on the proportion of some maximum used.
     """
 
@@ -181,18 +181,24 @@ class RateOfChangeTransformer(ScalingTransformer):
         self.cache = {}
         self.scale = self.scale or '1'
 
-    def handle_sample(self, context, s):
+    def handle_sample(self, s):
         """Handle a sample, converting if necessary."""
         LOG.debug('handling sample %s', s)
         key = s.name + s.resource_id
         prev = self.cache.get(key)
         timestamp = timeutils.parse_isotime(s.timestamp)
-        self.cache[key] = (s.volume, timestamp)
+        self.cache[key] = (s.volume, timestamp, s.monotonic_time)
 
         if prev:
             prev_volume = prev[0]
             prev_timestamp = prev[1]
-            time_delta = timeutils.delta_seconds(prev_timestamp, timestamp)
+            prev_monotonic_time = prev[2]
+            if (prev_monotonic_time is not None and
+                    s.monotonic_time is not None):
+                # NOTE(sileht): Prefer high precision timer
+                time_delta = s.monotonic_time - prev_monotonic_time
+            else:
+                time_delta = timeutils.delta_seconds(prev_timestamp, timestamp)
             # disallow violations of the arrow of time
             if time_delta < 0:
                 LOG.warning(_('dropping out of time order sample: %s'), (s,))
@@ -293,7 +299,7 @@ class AggregatorTransformer(ScalingTransformer):
         # NOTE(sileht): it assumes, a meter always have the same unit/type
         return "%s-%s-%s" % (s.name, s.resource_id, non_aggregated_keys)
 
-    def handle_sample(self, context, sample_):
+    def handle_sample(self, sample_):
         if not self.initial_timestamp:
             self.initial_timestamp = timeutils.parse_isotime(sample_.timestamp)
 
@@ -317,7 +323,7 @@ class AggregatorTransformer(ScalingTransformer):
                     setattr(self.samples[key], field,
                             getattr(sample_, field))
 
-    def flush(self, context):
+    def flush(self):
         if not self.initial_timestamp:
             return []
 
