@@ -6,9 +6,9 @@
 # [[local|localrc]]
 # enable_plugin ceilometer git://git.openstack.org/openstack/ceilometer
 #
-# By default all ceilometer services are started (see
-# devstack/settings). To disable a specific service use the
-# disable_service function.
+# By default all ceilometer services are started (see devstack/settings)
+# except for the ceilometer-aipmi service. To disable a specific service
+# use the disable_service function.
 #
 # NOTE: Currently, there are two ways to get the IPMI based meters in
 # OpenStack. One way is to configure Ironic conductor to report those meters
@@ -17,13 +17,15 @@
 # functionality. So in order to do so, users need to set the option of
 # conductor.send_sensor_data to true in the ironic.conf configuration file
 # for the Ironic conductor service, and also enable the
-# ceilometer-anotification service. If you do this disable the IPMI
-# polling agent:
-#
-# disable_service ceilometer-aipmi
+# ceilometer-anotification service.
 #
 # The other way is to use Ceilometer ipmi agent only to get the IPMI based
-# meters. To avoid duplicated meters, users need to make sure to set the
+# meters. To make use of the Ceilometer ipmi agent, it must be explicitly
+# enabled with the following setting:
+#
+# enable_service ceilometer-aipmi
+#
+# To avoid duplicated meters, users need to make sure to set the
 # option of conductor.send_sensor_data to false in the ironic.conf
 # configuration file if the node on which Ceilometer ipmi agent is running
 # is also managed by Ironic.
@@ -32,16 +34,13 @@
 # of Ceilometer (see within for additional settings):
 #
 #   CEILOMETER_PIPELINE_INTERVAL:  Seconds between pipeline processing runs. Default 600.
-#   CEILOMETER_BACKEND:            Database backend (e.g. 'mysql', 'mongodb', 'es')
+#   CEILOMETER_BACKEND:            Database backend (e.g. 'mysql', 'mongodb', 'gnocchi', 'none')
 #   CEILOMETER_COORDINATION_URL:   URL for group membership service provided by tooz.
-#   CEILOMETER_EVENTS:             Set to True to enable event collection
 #   CEILOMETER_EVENT_ALARM:        Set to True to enable publisher for event alarming
 
 # Save trace setting
 XTRACE=$(set +o | grep xtrace)
 set -o xtrace
-# TODO(liusheng) Temporarily add this to avoid integration test failue, see bug1548634
-export SERVICE_TENANT_NAME=$SERVICE_PROJECT_NAME
 
 # Support potential entry-points console scripts in VENV or not
 if [[ ${USE_VENV} = True ]]; then
@@ -139,11 +138,6 @@ function _ceilometer_prepare_storage_backend {
         pip_install_gr pymongo
         _ceilometer_install_mongodb
     fi
-
-    if [ "$CEILOMETER_BACKEND" = 'es' ] ; then
-        ${TOP_DIR}/pkg/elasticsearch.sh download
-        ${TOP_DIR}/pkg/elasticsearch.sh install
-    fi
 }
 
 
@@ -163,23 +157,29 @@ function _ceilometer_prepare_virt_drivers {
 
 
 # Create ceilometer related accounts in Keystone
-function _ceilometer_create_accounts {
+function ceilometer_create_accounts {
+    # At this time, the /etc/openstack/clouds.yaml is available,
+    # we could leverage that by setting OS_CLOUD
+    OLD_OS_CLOUD=$OS_CLOUD
+    export OS_CLOUD='devstack-admin'
+
+    create_service_user "ceilometer" "admin"
+
     if is_service_enabled ceilometer-api; then
-
-        create_service_user "ceilometer" "admin"
-
         get_or_create_service "ceilometer" "metering" "OpenStack Telemetry Service"
         get_or_create_endpoint "metering" \
             "$REGION_NAME" \
             "$(ceilometer_service_url)" \
             "$(ceilometer_service_url)" \
             "$(ceilometer_service_url)"
-
-        if is_service_enabled swift; then
-            # Ceilometer needs ResellerAdmin role to access Swift account stats.
-            get_or_add_user_project_role "ResellerAdmin" "ceilometer" $SERVICE_PROJECT_NAME
-        fi
     fi
+
+    if is_service_enabled swift; then
+        # Ceilometer needs ResellerAdmin role to access Swift account stats.
+        get_or_add_user_project_role "ResellerAdmin" "ceilometer" $SERVICE_PROJECT_NAME
+    fi
+
+    export OS_CLOUD=$OLD_OS_CLOUD
 }
 
 # Activities to do before ceilometer has been installed.
@@ -196,12 +196,10 @@ function _ceilometer_cleanup_apache_wsgi {
     fi
 }
 
-function _drop_database {
+function _ceilometer_drop_database {
     if is_service_enabled ceilometer-collector ceilometer-api ; then
         if [ "$CEILOMETER_BACKEND" = 'mongodb' ] ; then
             mongo ceilometer --eval "db.dropDatabase();"
-        elif [ "$CEILOMETER_BACKEND" = 'es' ] ; then
-            curl -XDELETE "localhost:9200/events_*"
         fi
     fi
 }
@@ -210,19 +208,16 @@ function _drop_database {
 # from previous runs that a clean run would need to clean up
 function cleanup_ceilometer {
     _ceilometer_cleanup_apache_wsgi
-    _drop_database
+    _ceilometer_drop_database
     sudo rm -f "$CEILOMETER_CONF_DIR"/*
     sudo rmdir "$CEILOMETER_CONF_DIR"
-    if is_service_enabled ceilometer-api && [ "$CEILOMETER_USE_MOD_WSGI" == "False" ]; then
-        sudo rm -f "$CEILOMETER_API_LOG_DIR"/*
-        sudo rmdir "$CEILOMETER_API_LOG_DIR"
-    fi
 }
 
 # Set configuraiton for cache backend.
 # NOTE(cdent): This currently only works for redis. Still working
 # out how to express the other backends.
 function _ceilometer_configure_cache_backend {
+    iniset $CEILOMETER_CONF cache enabled True
     iniset $CEILOMETER_CONF cache backend $CEILOMETER_CACHE_BACKEND
     iniset $CEILOMETER_CONF cache backend_argument url:$CEILOMETER_CACHE_URL
     iniadd_literal $CEILOMETER_CONF cache backend_argument distributed_lock:True
@@ -235,29 +230,32 @@ function _ceilometer_configure_cache_backend {
 
 # Set configuration for storage backend.
 function _ceilometer_configure_storage_backend {
-    if [ "$CEILOMETER_BACKEND" = 'mysql' ] || [ "$CEILOMETER_BACKEND" = 'postgresql' ] ; then
-        iniset $CEILOMETER_CONF database event_connection $(database_connection_url ceilometer)
+    # clear all old dispatchers since iniset/iniadd functions don't support
+    # multivalue options well.
+    inidelete $CEILOMETER_CONF DEFAULT meter_dispatchers
+    inidelete $CEILOMETER_CONF DEFAULT event_dispatchers
+
+    if [ "$CEILOMETER_BACKEND" = 'none' ] ; then
+        # It's ok for the backend to be 'none', if panko is enabled. We do not
+        # combine this condition with the outer if statement, so that the else
+        # clause below is not executed.
+        if ! is_service_enabled panko-api; then
+            echo_summary "All Ceilometer backends seems disabled, set \$CEILOMETER_BACKEND to select one."
+        fi
+    elif [ "$CEILOMETER_BACKEND" = 'mysql' ] || [ "$CEILOMETER_BACKEND" = 'postgresql' ] ; then
+        iniadd $CEILOMETER_CONF DEFAULT meter_dispatchers database
         iniset $CEILOMETER_CONF database metering_connection $(database_connection_url ceilometer)
-    elif [ "$CEILOMETER_BACKEND" = 'es' ] ; then
-        # es is only supported for events. we will use sql for metering.
-        iniset $CEILOMETER_CONF database event_connection es://localhost:9200
-        iniset $CEILOMETER_CONF database metering_connection $(database_connection_url ceilometer)
-        ${TOP_DIR}/pkg/elasticsearch.sh start
     elif [ "$CEILOMETER_BACKEND" = 'mongodb' ] ; then
-        iniset $CEILOMETER_CONF database event_connection mongodb://localhost:27017/ceilometer
+        iniadd $CEILOMETER_CONF DEFAULT meter_dispatchers database
         iniset $CEILOMETER_CONF database metering_connection mongodb://localhost:27017/ceilometer
     elif [ "$CEILOMETER_BACKEND" = 'gnocchi' ] ; then
-        gnocchi_url=$(gnocchi_service_url)
-        iniset $CEILOMETER_CONF DEFAULT meter_dispatchers gnocchi
-        # FIXME(sileht): We shouldn't load event_dispatchers if store_event is False
-        iniset $CEILOMETER_CONF DEFAULT event_dispatchers ""
-        iniset $CEILOMETER_CONF notification store_events False
+        iniadd $CEILOMETER_CONF DEFAULT meter_dispatchers gnocchi
+        iniadd $CEILOMETER_CONF DEFAULT event_dispatchers gnocchi
         # NOTE(gordc): set higher retry in case gnocchi is started after ceilometer on a slow machine
         iniset $CEILOMETER_CONF storage max_retries 20
         # NOTE(gordc): set batching to better handle recording on a slow machine
         iniset $CEILOMETER_CONF collector batch_size 50
         iniset $CEILOMETER_CONF collector batch_timeout 5
-        iniset $CEILOMETER_CONF dispatcher_gnocchi url $gnocchi_url
         iniset $CEILOMETER_CONF dispatcher_gnocchi archive_policy ${GNOCCHI_ARCHIVE_POLICY}
         if is_service_enabled swift && [[ "$GNOCCHI_STORAGE_BACKEND" = 'swift' ]] ; then
             iniset $CEILOMETER_CONF dispatcher_gnocchi filter_service_activity "True"
@@ -268,7 +266,13 @@ function _ceilometer_configure_storage_backend {
     else
         die $LINENO "Unable to configure unknown CEILOMETER_BACKEND $CEILOMETER_BACKEND"
     fi
-    _drop_database
+
+    # configure panko
+    if is_service_enabled panko-api; then
+        iniadd $CEILOMETER_CONF DEFAULT event_dispatchers panko
+    fi
+
+    _ceilometer_drop_database
 }
 
 # Configure Ceilometer
@@ -278,13 +282,11 @@ function configure_ceilometer {
 
     iniset_rpc_backend ceilometer $CEILOMETER_CONF
 
-    iniset $CEILOMETER_CONF DEFAULT notification_topics "$CEILOMETER_NOTIFICATION_TOPICS"
-    iniset $CEILOMETER_CONF DEFAULT verbose True
+    iniset $CEILOMETER_CONF oslo_messaging_notifications topics "$CEILOMETER_NOTIFICATION_TOPICS"
     iniset $CEILOMETER_CONF DEFAULT debug "$ENABLE_DEBUG_LOG_LEVEL"
 
     if [[ -n "$CEILOMETER_COORDINATION_URL" ]]; then
         iniset $CEILOMETER_CONF coordination backend_url $CEILOMETER_COORDINATION_URL
-        iniset $CEILOMETER_CONF compute workload_partitioning True
         iniset $CEILOMETER_CONF notification workload_partitioning True
         iniset $CEILOMETER_CONF notification workers $API_WORKERS
     fi
@@ -299,15 +301,13 @@ function configure_ceilometer {
     # with rootwrap installation done elsewhere and also clobber
     # ceilometer.conf settings that have already been made.
     # Anyway, explicit is better than implicit.
-    for conffile in policy.json api_paste.ini pipeline.yaml \
-                    event_definitions.yaml event_pipeline.yaml \
-                    gnocchi_resources.yaml; do
+    for conffile in policy.json api_paste.ini pipeline.yaml polling.yaml \
+                    event_definitions.yaml event_pipeline.yaml; do
         cp $CEILOMETER_DIR/etc/ceilometer/$conffile $CEILOMETER_CONF_DIR
     done
-    iniset $CEILOMETER_CONF oslo_policy policy_file $CEILOMETER_CONF_DIR/policy.json
 
     if [ "$CEILOMETER_PIPELINE_INTERVAL" ]; then
-        sed -i "s/interval:.*/interval: ${CEILOMETER_PIPELINE_INTERVAL}/" $CEILOMETER_CONF_DIR/pipeline.yaml
+        sed -i "s/interval:.*/interval: ${CEILOMETER_PIPELINE_INTERVAL}/" $CEILOMETER_CONF_DIR/polling.yaml
     fi
     if [ "$CEILOMETER_EVENT_ALARM" == "True" ]; then
         if ! grep -q '^ *- notifier://?topic=alarm.all$' $CEILOMETER_CONF_DIR/event_pipeline.yaml; then
@@ -328,8 +328,6 @@ function configure_ceilometer {
 
     configure_auth_token_middleware $CEILOMETER_CONF ceilometer $CEILOMETER_AUTH_CACHE_DIR
 
-    iniset $CEILOMETER_CONF notification store_events $CEILOMETER_EVENTS
-
     # Configure storage
     if is_service_enabled ceilometer-collector ceilometer-api; then
         _ceilometer_configure_storage_backend
@@ -344,7 +342,6 @@ function configure_ceilometer {
     fi
 
     if is_service_enabled ceilometer-api && [ "$CEILOMETER_USE_MOD_WSGI" == "True" ]; then
-        iniset $CEILOMETER_CONF api pecan_debug "False"
         _ceilometer_config_apache_wsgi
     fi
 
@@ -356,16 +353,21 @@ function configure_ceilometer {
 
 # init_ceilometer() - Initialize etc.
 function init_ceilometer {
-    # Get ceilometer keystone settings in place
-    _ceilometer_create_accounts
     # Create cache dir
     sudo install -d -o $STACK_USER $CEILOMETER_AUTH_CACHE_DIR
     rm -f $CEILOMETER_AUTH_CACHE_DIR/*
 
-    if is_service_enabled ceilometer-collector ceilometer-api && is_service_enabled mysql postgresql ; then
-        if [ "$CEILOMETER_BACKEND" = 'mysql' ] || [ "$CEILOMETER_BACKEND" = 'postgresql' ] || [ "$CEILOMETER_BACKEND" = 'es' ] ; then
-            recreate_database ceilometer
-            $CEILOMETER_BIN_DIR/ceilometer-dbsync
+    if is_service_enabled ceilometer-collector ceilometer-api; then
+        if is_service_enabled mysql postgresql ; then
+            if [ "$CEILOMETER_BACKEND" = 'mysql' ] || [ "$CEILOMETER_BACKEND" = 'postgresql' ] || [ "$CEILOMETER_BACKEND" = 'es' ] ; then
+                recreate_database ceilometer
+                $CEILOMETER_BIN_DIR/ceilometer-upgrade --skip-gnocchi-resource-types
+            fi
+        fi
+        if is_service_enabled gnocchi ; then
+            if [ "$CEILOMETER_BACKEND" = 'gnocchi' ]; then
+                $CEILOMETER_BIN_DIR/ceilometer-upgrade --skip-metering-database
+            fi
         fi
     fi
 }
@@ -376,7 +378,7 @@ function init_ceilometer {
 # installed. The context is not active during preinstall (when it would
 # otherwise makes sense to do the backend services).
 function install_ceilometer {
-    if is_service_enabled ceilometer-acentral ceilometer-anotification ceilometer-alarm-evaluator ; then
+    if is_service_enabled ceilometer-acentral ceilometer-acompute ceilometer-anotification ; then
         _ceilometer_prepare_coordination
     fi
 
@@ -391,9 +393,6 @@ function install_ceilometer {
     install_ceilometerclient
     setup_develop $CEILOMETER_DIR
     sudo install -d -o $STACK_USER -m 755 $CEILOMETER_CONF_DIR
-    if is_service_enabled ceilometer-api && [ "$CEILOMETER_USE_MOD_WSGI" == "False" ]; then
-        sudo install -d -o $STACK_USER -m 755 $CEILOMETER_API_LOG_DIR
-    fi
 }
 
 # install_ceilometerclient() - Collect source and prepare
@@ -414,7 +413,7 @@ function start_ceilometer {
     run_process ceilometer-aipmi "$CEILOMETER_BIN_DIR/ceilometer-polling --polling-namespaces ipmi --config-file $CEILOMETER_CONF"
 
     if [[ "$CEILOMETER_USE_MOD_WSGI" == "False" ]]; then
-        run_process ceilometer-api "$CEILOMETER_BIN_DIR/ceilometer-api -d -v --log-dir=$CEILOMETER_API_LOG_DIR --config-file $CEILOMETER_CONF"
+        run_process ceilometer-api "$CEILOMETER_BIN_DIR/ceilometer-api --port $CEILOMETER_SERVICE_PORT"
     elif is_service_enabled ceilometer-api; then
         enable_apache_site ceilometer
         restart_apache_server
@@ -434,14 +433,6 @@ function start_ceilometer {
     if [[ "$VIRT_DRIVER" = 'vsphere' ]]; then
         run_process ceilometer-acompute "$CEILOMETER_BIN_DIR/ceilometer-polling --polling-namespaces compute --config-file $CEILOMETER_CONF"
     fi
-
-    # Only die on API if it was actually intended to be turned on
-    if is_service_enabled ceilometer-api; then
-        echo "Waiting for ceilometer-api to start..."
-        if ! wait_for_service $SERVICE_TIMEOUT $(ceilometer_service_url)/v2/; then
-            die $LINENO "ceilometer-api did not start"
-        fi
-    fi
 }
 
 # stop_ceilometer() - Stop running processes
@@ -469,11 +460,13 @@ if is_service_enabled ceilometer; then
         preinstall_ceilometer
     elif [[ "$1" == "stack" && "$2" == "install" ]]; then
         echo_summary "Installing Ceilometer"
-        # Use stack_install_service here to account for vitualenv
+        # Use stack_install_service here to account for virtualenv
         stack_install_service ceilometer
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         echo_summary "Configuring Ceilometer"
         configure_ceilometer
+        # Get ceilometer keystone settings in place
+        ceilometer_create_accounts
     elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
         echo_summary "Initializing Ceilometer"
         # Tidy base for ceilometer
